package mathvis.ws06.jtem.ode;

import de.jreality.geometry.QuadMeshFactory;
import de.jreality.scene.SceneGraphComponent;
import de.jreality.tools.DragEventTool;
import de.jreality.tools.PointDragEvent;
import de.jreality.tools.PointDragListener;

public class EditableHeightField implements PointDragListener {

	private SceneGraphComponent root;
	private QuadMeshFactory hff;
	
	double[][] values;
	private double dragZ;
	double y = .7;
	double z = .5;
	double[][] brush = {{0,0,1},{0,1,y},{0,-1,y},{1,0,y},{-1,0,y}
					   ,{1,1,z},{1,-1,z},{-1,1,z},{-1,-1,z}};
	
	private final int u, v;
	
	public EditableHeightField(int u, int v) {
		root = new SceneGraphComponent();
		root.setName("height field root");
		this.u=u;
		this.v=v;
		hff = new QuadMeshFactory();
		hff.setULineCount(u);
		hff.setVLineCount(v);
		hff.setClosedInUDirection(false);
		hff.setClosedInVDirection(false);
		values = new double[u*v][3];
		for (int j = 0; j < v; j++) {
			for (int i = 0 ; i < u; i++) {
				values[j*u+i][0]=i;
				values[j*u+i][1]=j;
			}
		}
		hff.setGenerateFaceNormals(true);
		hff.setGenerateVertexNormals(true);
		updateGeometry();
		root.setGeometry(hff.getGeometry());
		DragEventTool dragEventTool = new DragEventTool();
		root.addTool(dragEventTool);
		dragEventTool.addPointDragListener(this);
	}

	private void updateGeometry() {
		hff.setVertexCoordinates(values);
		hff.update();
	}

	public void pointDragEnd(PointDragEvent e) {
	}

	public void pointDragStart(PointDragEvent e) {
		dragZ = e.getZ();
	}

	public void pointDragged(PointDragEvent e) {
		int ind = e.getIndex();
		double z = e.getZ();
		
		int i = ind % u;
		int j = ind/u;
		
		double dz=z-dragZ;
		dragZ=z;
		
		for (int k = 0; k < brush.length; k++) {
			int ci=(int) (i+brush[k][0]);
			int cj=(int) (j+brush[k][1]);
			if (ci>=0 && cj>=0 && ci<u && cj<v) {
				values[cj*u+ci][2]+=brush[k][2]*dz;
			}
		}
		updateGeometry();
	}

	public SceneGraphComponent getRoot() {
		return root;
	}

	public double[] getValues() {
		double[] ret = new double[u*v];
		for (int j = 0; j < v; j++) {
			for (int i = 0 ; i < u; i++) {
				ret[j*u+i]=values[j*u+i][2];
			}
		}
		return ret;
	}

	public void setValues(double[] vals) {
		for (int j = 0; j < v; j++) {
			for (int i = 0 ; i < u; i++) {
				values[j*u+i][2]=vals[j*u+i];
			}
		}
		hff.setVertexCoordinates(values);
		hff.update();
	}
	
}
